@@ -1,7 +1,7 @@
 // samplv1.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@
 // forward declarations
 class samplv1_impl;
 class samplv1_sample;
+class samplv1_programs;
 
 
 //-------------------------------------------------------------------------
@@ -145,9 +146,12 @@ public:
 	void setParamPort(ParamIndex index, float *pfParam);
 	float *paramPort(ParamIndex index) const;
 
+	void setParamValue(ParamIndex index, float fValue);
+	float paramValue(ParamIndex index) const;
+
 	void reset();
 
-protected:
+	samplv1_programs *programs() const;
 
 	void process_midi(uint8_t *data, uint32_t size);
 	void process(float **ins, float **outs, uint32_t nframes);
