@@ -1,7 +1,7 @@
 // samplv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -22,8 +22,6 @@
 #include "samplv1widget.h"
 #include "samplv1_param.h"
 
-#include "samplv1_sample.h"
-
 #include "samplv1_sample.h"
 #include "samplv1_sched.h"
 
@@ -480,14 +478,10 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
-	samplv1widget_config *pConfig = samplv1widget_config::getInstance();
-	if (pConfig)
-		m_ui.helpUseNativeDialogsAction->setChecked(pConfig->bUseNativeDialogs);
-
 	// Menu actions
-	QObject::connect(m_ui.helpUseNativeDialogsAction,
+	QObject::connect(m_ui.helpConfigureAction,
 		SIGNAL(triggered(bool)),
-		SLOT(helpUseNativeDialogs(bool)));
+		SLOT(helpConfigure()));
 	QObject::connect(m_ui.helpAboutAction,
 		SIGNAL(triggered(bool)),
 		SLOT(helpAbout()));
@@ -497,8 +491,15 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	// Special sample update notifications (eg. reverse)
 	QObject::connect(m_sched_notifier,
-		SIGNAL(notify()),
-		SLOT(updateSampleNotify()));
+		SIGNAL(notify(int)),
+		SLOT(updateSchedNotify(int)));
+
+	// General knob/dial  behavior init...
+	samplv1_config *pConfig = samplv1_config::getInstance();
+	if (pConfig) {
+		samplv1widget_dial::setDialMode(
+			samplv1widget_dial::DialMode(pConfig->iKnobDialMode));
+	}
 
 	// Epilog.
 	// QWidget::adjustSize();
@@ -552,21 +553,18 @@ void samplv1widget::setParamValue (
 
 float samplv1widget::paramValue ( samplv1::ParamIndex index ) const
 {
-	float fParamValue = 0.0f;
+	float fValue = 0.0f;
 
 	samplv1widget_knob *pKnob = paramKnob(index);
 	if (pKnob) {
-		fParamValue = pKnob->value();
+		fValue = pKnob->value();
 	} else {
-		samplv1 *pSampl = instance();
-		if (pSampl) {
-			const float *pParamPort = pSampl->paramPort(index);
-			if (pParamPort)
-				fParamValue = *pParamPort;
-		}
+		samplv1_ui *pSamplUi = ui_instance();
+		if (pSamplUi)
+			fValue = pSamplUi->paramValue(index);
 	}
 
-	return fParamValue;
+	return fValue;
 }
 
 
@@ -593,14 +591,14 @@ void samplv1widget::paramChanged ( float fValue )
 // Update local tied widgets.
 void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 {
-	samplv1 *pSampl = instance();
-	if (pSampl == NULL)
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi == NULL)
 		return;
 
 	++m_iUpdate;
 
 	switch (index) {
-#if 0//--updateSampleNotify();
+#if 0//--updateSchedNotify(samplv1_sched::Sample);
 	case samplv1::GEN1_REVERSE: {
 		const bool bReverse = bool(fValue > 0.0f);
 		pSampl->setReverse(bReverse);
@@ -610,12 +608,12 @@ void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 #endif
 	case samplv1::GEN1_LOOP: {
 		const bool bLoop = bool(fValue > 0.0f);
-		pSampl->setLoop(bLoop);
+		pSamplUi->setLoop(bLoop);
 		m_ui.Gen1Sample->setLoop(bLoop);
-		m_ui.Gen1Sample->setLoopStart(pSampl->loopStart());
-		m_ui.Gen1Sample->setLoopEnd(pSampl->loopEnd());
+		m_ui.Gen1Sample->setLoopStart(pSamplUi->loopStart());
+		m_ui.Gen1Sample->setLoopEnd(pSamplUi->loopEnd());
 		m_ui.Gen1LoopRangeFrame->setEnabled(bLoop);
-		updateSampleLoop(pSampl->sample());
+		updateSampleLoop(pSamplUi->sample());
 		break;
 	}
 	case samplv1::DEL1_BPMSYNC:
@@ -633,11 +631,11 @@ void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 // Reset all param knobs to default values.
 void samplv1widget::resetParams (void)
 {
-	samplv1 *pSampl = instance();
-	if (pSampl == NULL)
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi == NULL)
 		return;
 
-	pSampl->reset();
+	pSamplUi->reset();
 
 	resetSwapParams();
 
@@ -700,15 +698,13 @@ void samplv1widget::updateParamValues (void)
 {
 	resetSwapParams();
 
-	samplv1 *pSampl = instance();
+	samplv1_ui *pSamplUi = ui_instance();
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		samplv1::ParamIndex index = samplv1::ParamIndex(i);
-		float fValue = samplv1_param::paramDefaultValue(index);
-		const float *pfParamPort
-			= (pSampl ? pSampl->paramPort(index) : NULL);
-		if (pfParamPort)
-			fValue = *pfParamPort;
+		const float fValue = (pSamplUi
+			? pSamplUi->paramValue(index)
+			: samplv1_param::paramDefaultValue(index));
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
 	//	updateParamEx(index, fValue);
@@ -788,21 +784,11 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 	resetParamKnobs();
 	resetParamValues();
 
-	samplv1 *pSampl = instance();
-	if (pSampl == NULL)
-		return;
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi)
+		samplv1_param::loadPreset(pSamplUi->instance(), sFilename);
 
-	samplv1_param::loadPreset(pSampl, sFilename);
-
-	updateSample(pSampl->sample());
-	updateParamValues();
-
-	const QString& sPreset
-		= QFileInfo(sFilename).completeBaseName();
-
-	m_ui.Preset->setPreset(sPreset);
-	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
-	updateDirtyPreset(false);
+	updateLoadPreset(QFileInfo(sFilename).completeBaseName());
 }
 
 
@@ -812,7 +798,9 @@ void samplv1widget::savePreset ( const QString& sFilename )
 	qDebug("samplv1widget::savePreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
 
-	samplv1_param::savePreset(instance(), sFilename);
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi)
+		samplv1_param::savePreset(pSamplUi->instance(), sFilename);
 
 	const QString& sPreset
 		= QFileInfo(sFilename).completeBaseName();
@@ -835,7 +823,7 @@ void samplv1widget::clearSample (void)
 // Sample file loader slot.
 void samplv1widget::loadSample ( const QString& sFilename )
 {
-	loadSampleFile(sFilename);
+	loadSampleFile(QFileInfo(sFilename).canonicalFilePath());
 
 	m_ui.StatusBar->showMessage(tr("Load sample: %1").arg(sFilename), 5000);
 	updateDirtyPreset(true);
@@ -849,19 +837,6 @@ void samplv1widget::openSample (void)
 }
 
 
-// Sample updater slot.
-void samplv1widget::updateSampleNotify (void)
-{
-#ifdef CONFIG_DEBUG
-	qDebug("samplv1widget::updateSampleNotify()");
-#endif
-
-	samplv1 *pSampl = instance();
-	if (pSampl)
-		updateSample(pSampl->sample());
-}
-
-
 // Sample file reset.
 void samplv1widget::clearSampleFile (void)
 {
@@ -869,9 +844,9 @@ void samplv1widget::clearSampleFile (void)
 	qDebug("samplv1widget::clearSampleFile()");
 #endif
 
-	samplv1 *pSampl = instance();
-	if (pSampl)
-		pSampl->setSampleFile(0);
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi)
+		pSamplUi->setSampleFile(0);
 
 	updateSample(0);
 }
@@ -884,10 +859,10 @@ void samplv1widget::loadSampleFile ( const QString& sFilename )
 	qDebug("samplv1widget::loadSampleFile(\"%s\")", sFilename.toUtf8().constData());
 #endif
 
-	samplv1 *pSampl = instance();
-	if (pSampl) {
-		pSampl->setSampleFile(sFilename.toUtf8().constData());
-		updateSample(pSampl->sample());
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
+		pSamplUi->setSampleFile(sFilename.toUtf8().constData());
+		updateSample(pSamplUi->sample());
 	}
 }
 
@@ -942,12 +917,12 @@ void samplv1widget::loopRangeChanged (void)
 		return;
 
 	++m_iUpdate;
-	samplv1 *pSampl = instance();
-	if (pSampl) {
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
 		const uint32_t iLoopStart = m_ui.Gen1Sample->loopStart();
 		const uint32_t iLoopEnd = m_ui.Gen1Sample->loopEnd();
-		pSampl->setLoopRange(iLoopStart, iLoopEnd);
-		updateSampleLoop(pSampl->sample(), true);
+		pSamplUi->setLoopRange(iLoopStart, iLoopEnd);
+		updateSampleLoop(pSamplUi->sample(), true);
 	}
 	--m_iUpdate;
 }
@@ -960,13 +935,13 @@ void samplv1widget::loopStartChanged (void)
 		return;
 
 	++m_iUpdate;
-	samplv1 *pSampl = instance();
-	if (pSampl) {
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
 		const uint32_t iLoopStart = m_ui.Gen1LoopStartSpinBox->value();
-		const uint32_t iLoopEnd = pSampl->loopEnd();
-		pSampl->setLoopRange(iLoopStart, iLoopEnd);
+		const uint32_t iLoopEnd = pSamplUi->loopEnd();
+		pSamplUi->setLoopRange(iLoopStart, iLoopEnd);
 		m_ui.Gen1Sample->setLoopStart(iLoopStart);
-		updateSampleLoop(pSampl->sample(), true);
+		updateSampleLoop(pSamplUi->sample(), true);
 	}
 	--m_iUpdate;
 }
@@ -979,13 +954,13 @@ void samplv1widget::loopEndChanged (void)
 		return;
 
 	++m_iUpdate;
-	samplv1 *pSampl = instance();
-	if (pSampl) {
-		const uint32_t iLoopStart = pSampl->loopStart();
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
+		const uint32_t iLoopStart = pSamplUi->loopStart();
 		const uint32_t iLoopEnd = m_ui.Gen1LoopEndSpinBox->value();
-		pSampl->setLoopRange(iLoopStart, iLoopEnd);
+		pSamplUi->setLoopRange(iLoopStart, iLoopEnd);
 		m_ui.Gen1Sample->setLoopEnd(iLoopEnd);
-		updateSampleLoop(pSampl->sample(), true);
+		updateSampleLoop(pSamplUi->sample(), true);
 	}
 	--m_iUpdate;
 }
@@ -1028,15 +1003,12 @@ void samplv1widget::bpmSyncChanged (void)
 		return;
 
 	++m_iUpdate;
-	samplv1 *pSampl = instance();
-	if (pSampl) {
-		float *pBpmSync = pSampl->paramPort(samplv1::DEL1_BPMSYNC);
-		if (pBpmSync) {
-			const bool bBpmSync0 = (*pBpmSync > 0.0f);
-			const bool bBpmSync1 = m_ui.Del1BpmKnob->isSpecialValue();
-			if ((bBpmSync1 && !bBpmSync0) || (!bBpmSync1 && bBpmSync0))
-				*pBpmSync = (bBpmSync1 ? 1.0f : 0.0f);
-		}
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
+		const bool bBpmSync0 = (pSamplUi->paramValue(samplv1::DEL1_BPMSYNC) > 0.0f);
+		const bool bBpmSync1 = m_ui.Del1BpmKnob->isSpecialValue();
+		if ((bBpmSync1 && !bBpmSync0) || (!bBpmSync1 && bBpmSync0))
+			pSamplUi->setParamValue(samplv1::DEL1_BPMSYNC, (bBpmSync1 ? 1.0f : 0.0f));
 	}
 	--m_iUpdate;
 }
@@ -1048,15 +1020,15 @@ void samplv1widget::contextMenuRequest ( const QPoint& pos )
 	QMenu menu(this);
 	QAction *pAction;
 
-	samplv1 *pSampl = instance();
+	samplv1_ui *pSamplUi = ui_instance();
 	const char *pszSampleFile = NULL;
-	if (pSampl)
-		pszSampleFile = pSampl->sampleFile();
+	if (pSamplUi)
+		pszSampleFile = pSamplUi->sampleFile();
 
 	pAction = menu.addAction(
 		QIcon(":/images/fileOpen.png"),
 		tr("Open Sample..."), this, SLOT(openSample()));
-	pAction->setEnabled(pSampl != NULL);
+	pAction->setEnabled(pSamplUi != NULL);
 	menu.addSeparator();
 	pAction = menu.addAction(
 		tr("Reset"), this, SLOT(clearSample()));
@@ -1066,14 +1038,60 @@ void samplv1widget::contextMenuRequest ( const QPoint& pos )
 }
 
 
-// Menu actions.
-void samplv1widget::helpUseNativeDialogs ( bool bOn )
+// Preset status updater.
+void samplv1widget::updateLoadPreset ( const QString& sPreset )
 {
-	samplv1widget_config *pConfig = samplv1widget_config::getInstance();
-	if (pConfig) {
-		pConfig->bUseNativeDialogs = bOn;
-		pConfig->bDontUseNativeDialogs = !pConfig->bUseNativeDialogs;
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi)
+		updateSample(pSamplUi->sample());
+
+	updateParamValues();
+
+	m_ui.Preset->setPreset(sPreset);
+	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
+	updateDirtyPreset(false);
+}
+
+
+// Notification updater.
+void samplv1widget::updateSchedNotify ( int stype )
+{
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi == NULL)
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("samplv1widget::updateSchedNotify(%d)", stype);
+#endif
+
+	switch (samplv1_sched::Type(stype)) {
+	case samplv1_sched::Programs: {
+		samplv1_programs *pPrograms = pSamplUi->programs();
+		samplv1_programs::Prog *pProg = pPrograms->current_prog();
+		if (pProg) updateLoadPreset(pProg->name());
+		break;
 	}
+	case samplv1_sched::Sample:
+		updateSample(pSamplUi->sample());
+		// Fall thru...
+	default:
+		break;
+	}
+}
+
+
+// Menu actions.
+void samplv1widget::helpConfigure (void)
+{
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi == NULL)
+		return;
+
+	samplv1widget_config form(this);
+
+	// Set programs database...
+	form.setPrograms(pSamplUi->programs());
+	form.exec();
 }
 
 
