@@ -1,7 +1,7 @@
 // samplv1widget_sample.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -20,8 +20,8 @@
 *****************************************************************************/
 
 #include "samplv1widget_sample.h"
-#include "samplv1widget_config.h"
 
+#include "samplv1_config.h"
 #include "samplv1_sample.h"
 
 #include <sndfile.h>
@@ -30,9 +30,13 @@
 
 #include <QApplication>
 #include <QFileDialog>
+#include <QMimeData>
+#include <QDrag>
 #include <QUrl>
 
 #include <QMouseEvent>
+#include <QDragEnterEvent>
+#include <QDropEvent>
 
 #include <QToolTip>
 
@@ -49,6 +53,8 @@ samplv1widget_sample::samplv1widget_sample (
 	QFrame::setFocusPolicy(Qt::ClickFocus);
 	QFrame::setMinimumSize(QSize(480, 80));
 
+	QFrame::setAcceptDrops(true);
+
 	QFrame::setFrameShape(QFrame::Panel);
 	QFrame::setFrameShadow(QFrame::Sunken);
 
@@ -190,13 +196,6 @@ uint32_t samplv1widget_sample::loopEnd (void) const
 }
 
 
-// Widget resize handler.
-void samplv1widget_sample::resizeEvent ( QResizeEvent * )
-{
-//	setSample(m_pSample);	-- reset polygon...
-}
-
-
 // Sanitizer helper;
 int samplv1widget_sample::safeX ( int x ) const
 {
@@ -211,22 +210,27 @@ int samplv1widget_sample::safeX ( int x ) const
 }
 
 
+// Widget resize handler.
+void samplv1widget_sample::resizeEvent ( QResizeEvent * )
+{
+//	setSample(m_pSample);	-- reset polygon...
+}
+
+
 // Mouse interaction.
 void samplv1widget_sample::mousePressEvent ( QMouseEvent *pMouseEvent )
 {
-	if (m_bLoop) {
-		if (pMouseEvent->button() == Qt::LeftButton) {
-			if (m_dragCursor == DragNone) {
-				m_dragState = DragStart;
-				m_posDrag = pMouseEvent->pos();
-			} else {
-				const int w = QFrame::width();
-				const uint32_t nframes = m_pSample->length();
-				if (nframes > 0) {
-					m_iDragStartX = safeX((m_iLoopStart * w) / nframes);
-					m_iDragEndX   = safeX((m_iLoopEnd   * w) / nframes);
-					m_dragState   = m_dragCursor;
-				}
+	if (pMouseEvent->button() == Qt::LeftButton) {
+		if (m_dragCursor == DragNone) {
+			m_dragState = DragStart;
+			m_posDrag = pMouseEvent->pos();
+		} else if (m_bLoop) {
+			const int w = QFrame::width();
+			const uint32_t nframes = m_pSample->length();
+			if (nframes > 0) {
+				m_iDragStartX = safeX((m_iLoopStart * w) / nframes);
+				m_iDragEndX   = safeX((m_iLoopEnd   * w) / nframes);
+				m_dragState   = m_dragCursor;
 			}
 		}
 	}
@@ -323,9 +327,21 @@ void samplv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 		if ((m_posDrag - pMouseEvent->pos()).manhattanLength()
 			> QApplication::startDragDistance()) {
 			// Start dragging alright...
-			m_dragState = m_dragCursor = DragSelect;
-			m_iDragStartX = m_iDragEndX = m_posDrag.x();
-			QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+			if (m_bLoop && (pMouseEvent->modifiers()
+				& (Qt::ShiftModifier | Qt::ControlModifier))) {
+				m_dragState = m_dragCursor = DragSelect;
+				m_iDragStartX = m_iDragEndX = m_posDrag.x();
+				QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+			} else if (m_pSample && m_pSample->filename()) {
+				QList<QUrl> urls;
+				urls.append(QUrl::fromLocalFile(m_pSample->filename()));
+				QMimeData *pMimeData = new QMimeData();
+				pMimeData->setUrls(urls);;
+				QDrag *pDrag = new QDrag(this);
+				pDrag->setMimeData(pMimeData);
+				pDrag->exec(Qt::CopyAction);
+				resetDragState();
+			}
 		}
 		// Fall thru...
 	default:
@@ -379,7 +395,6 @@ void samplv1widget_sample::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		break;
 	}
 
-
 	resetDragState();
 }
 
@@ -405,6 +420,27 @@ void samplv1widget_sample::keyPressEvent ( QKeyEvent *pKeyEvent )
 }
 
 
+// Drag-n-drop (more of the later) support.
+void samplv1widget_sample::dragEnterEvent ( QDragEnterEvent *pDragEnterEvent )
+{
+	if (pDragEnterEvent->source() != this
+		&& pDragEnterEvent->mimeData()->hasUrls())
+		pDragEnterEvent->acceptProposedAction();
+}
+
+
+void samplv1widget_sample::dropEvent ( QDropEvent *pDropEvent )
+{
+	const QMimeData *pMimeData = pDropEvent->mimeData();
+	if (pMimeData->hasUrls()) {
+		const QString& sFilename
+			= QListIterator<QUrl>(pMimeData->urls()).peekNext().toLocalFile();
+		if (!sFilename.isEmpty())
+			emit loadSampleFile(sFilename);
+	}
+}
+
+
 // Reset drag/select state.
 void samplv1widget_sample::resetDragState (void)
 {
@@ -474,7 +510,7 @@ void samplv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 	} else {
 		painter.setPen(pal.midlight().color());
 		painter.drawText(rect, Qt::AlignCenter,
-			tr("(double-click to load new sample...)"));
+			tr("(double-click or drop to load new sample...)"));
 	}
 
 	painter.end();
@@ -486,7 +522,7 @@ void samplv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 // Browse for a new sample.
 void samplv1widget_sample::openSample (void)
 {
-	samplv1widget_config *pConfig = samplv1widget_config::getInstance();
+	samplv1_config *pConfig = samplv1_config::getInstance();
 	if (pConfig == NULL)
 		return;
 
